import pytest
import os.path
from _fedmod.cli import ModtoolsCLI
from _fedmod.module_generator import ModuleGenerator

def _generate_modulemd(rpms):
    cmd_input = list(['rpm2module'])
    cmd_input.extend(rpms)
    cli = ModtoolsCLI(cmd_input)
    mg = ModuleGenerator(cli.pkgs)
    if len(rpms) == 1:
        output_fname = rpms[0] + '.yaml'
    else:
        output_fname = 'modulemd-output.yaml'
    mg.run(output_fname)
    return mg, output_fname


class TestSinglePackageInput(object):

    def setup(self):
        self.input_rpms = input_rpms = ('grep',)
        self.md, self.output_fname = _generate_modulemd(input_rpms)

    def teardown(self):
        os.remove(self.output_fname)

    def test_generated_modulemd_file(self):
        # File exists with expected name
        assert os.path.isfile(self.input_rpms[0] + '.yaml')

        # Check details of generated module metadata
        modmd = self.md.mmd

        # Expected description for 'grep'
        assert modmd.summary == "Pattern matching utilities"
        assert (modmd.description == 'The GNU versions of commonly used grep utilities. Grep searches through\n' +
                'textual input for lines which contain a match to a specified pattern and then\n' +
                'prints the matching lines. GNU\'s grep utilities include grep, egrep and fgrep.\n\n' +
                'GNU grep is needed by many scripts, so it shall be installed on every system.')

        # Expected licenses for 'grep'
        assert len(modmd.module_licenses) == 1
        assert sorted(modmd.module_licenses) == sorted(['MIT'])
        assert len(modmd.content_licenses) == 1
        assert sorted(modmd.content_licenses) == sorted(['GPLv3+'])

        # Only given modules are listed in the public API
        assert sorted(modmd.api.rpms) == sorted(self.input_rpms)

        # Expected components for grep
        assert set(modmd.components.rpms) == set(self.input_rpms)

        # Expected module dependencies for grep
        assert set(modmd.buildrequires) == set()
        assert set(modmd.requires) == set(('platform',))



class TestMultiplePackageInput(object):

    def setup(self):
        self.input_rpms = input_rpms = ('grep', 'haproxy')
        self.md, self.output_fname = _generate_modulemd(input_rpms)

    def teardown(self):
        os.remove(self.output_fname)

    def test_generated_modulemd_file(self):
        # File exists with expected name
        assert os.path.isfile('modulemd-output.yaml')

        # Check details of generated module metadata
        modmd = self.md.mmd

        # Can't generate descriptive metadata when given multiple RPMs
        assert modmd.summary == ""
        assert modmd.description == ""

        # Expected licenses for grep + haproxy
        assert len(modmd.module_licenses) == 1
        assert sorted(modmd.module_licenses) == sorted(['MIT'])
        assert len(modmd.content_licenses) == 0 # This doesn't seem right...

        # Only given modules are listed in the public API
        assert sorted(modmd.api.rpms) == sorted(self.input_rpms)

        # Expected components for grep + haproxy
        expected_components = set(self.input_rpms)
        assert set(modmd.components.rpms) == expected_components

        # Expected module dependencies for grep + haproxy
        assert set(modmd.buildrequires) == set(('platform',))
        assert set(modmd.requires) == set(('platform',))
